@@ -12,7 +12,6 @@
 #include <kwin.h>
 #include <kdebug.h>
 #include <kcmdlineargs.h>
-#include <kurl.h>
 
 #include "application.h"
 #include "toplevel.h"
@@ -32,18 +31,7 @@ Application::~Application()
 int Application::newInstance()
 {
   kdDebug(5004) << "Application::newInstance()" << endl;
-
-  if (mainWidget()) {
-    KWin::setOnDesktop(mainWidget()->winId(),KWin::currentDesktop());
-    KWin::setActiveWindow(mainWidget()->winId());
-  } else {
-    if (isRestored()) {
-      RESTORE(TopLevel);
-    } else {
-      TopLevel* top = new TopLevel();
-      top->show();
-    }
-  }
+  KUniqueApplication::newInstance();
 
   // process parameters...
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
