@@ -2,7 +2,7 @@
 *                                transfer.cpp
 *                             -------------------
 *
-*    Revision     : $Id: transfer.cpp,v 1.48.2.2 2004/03/17 11:32:31 lukas Exp $
+*    Revision     : $Id: transfer.cpp,v 1.57.2.1 2004/08/09 18:19:10 tokoe Exp $
 *    begin        : Tue Jan 29 2002
 *    copyright    : (C) 2002 by Patrick Charbonnier
 *                 : Based On Caitoo v.0.7.3 (c) 1998 - 2000, Matej Koss
@@ -24,6 +24,7 @@
  *
  ***************************************************************************/
 
+#include <qheader.h>
 
 #include <kurl.h>
 #include <kmessagebox.h>
@@ -52,9 +53,10 @@
 extern Settings ksettings;
 
 
-Transfer::Transfer(TransferList * _view, const KURL & _src, const KURL & _dest)
+Transfer::Transfer(TransferList * _view, const KURL & _src, const KURL & _dest, const uint _id)
     : QObject( _view ),
-      KListViewItem(_view)
+      KListViewItem(_view),
+      dlgIndividual( 0 )
 {
     sDebugIn << endl;
 
@@ -62,22 +64,24 @@ Transfer::Transfer(TransferList * _view, const KURL & _src, const KURL & _dest)
     dest = _dest;
 
     view = _view;
-    init();
+    init(_id);
 
     sDebugOut << endl;
 }
 
 
-Transfer::Transfer(TransferList * _view, Transfer * after, const KURL & _src, const KURL & _dest)
+Transfer::Transfer(TransferList * _view, Transfer * after, const KURL & _src, const KURL & _dest, const uint _id)
     : QObject( _view ),
-      KListViewItem(_view, (QListViewItem *) after)
+      KListViewItem(_view, (QListViewItem *) after),
+      src(_src), dest(_dest), view(_view),
+      dlgIndividual( 0 )
 {
     sDebugIn << endl;
 
     view = _view;
     src = _src;
     dest = _dest;
-    init();
+    init(_id);
 
     sDebugOut << endl;
 }
@@ -88,21 +92,21 @@ Transfer::~Transfer()
     sDebugIn << endl;
 
     synchronousAbort();
-
     delete dlgIndividual;
+
     sDebugOut << endl;
 }
 
 
 void
-Transfer::init()
+Transfer::init(const uint _id)
 {
     sDebugIn << endl;
 
     totalSize = 0;
     processedSize = 0;
     percent = 0;
-    id = 0;
+    id = _id;
     m_pSlave = new Slave(this, src, dest);
     canResume = false;
     startTime = QDateTime::currentDateTime();
@@ -149,10 +153,6 @@ Transfer::init()
     // m_paDock = new KAction(i18n("&Dock"),"tool_dock", 0, this,SLOT(slotRequestDelay()), this, "dockIndividual");
 
     // setup individual transfer dialog
-    dlgIndividual = new DlgIndividual(this);
-    if (ksettings.b_iconifyIndividual) {
-        KWin::iconifyWindow( dlgIndividual->winId() );
-    }
 
     sDebugOut << endl;
 }
@@ -162,7 +162,14 @@ void Transfer::synchronousAbort()
 {
     if ( m_pSlave )
     {
-        m_pSlave->Op(Slave::KILL);
+        if ( m_pSlave->running() )
+        {
+            m_pSlave->Op(Slave::KILL);
+            m_pSlave->wait();
+        }
+
+        if ( m_pSlave->running() )
+            m_pSlave->terminate();
 
         delete m_pSlave;
         m_pSlave = 0L;
@@ -181,7 +188,8 @@ void Transfer::slotUpdateActions()
         m_paResume->setEnabled(false);
         m_paPause->setEnabled(false);
         m_paRestart->setEnabled(false);
-        dlgIndividual->update();
+        if(dlgIndividual)
+			dlgIndividual->update();
         return;
     }
 
@@ -240,7 +248,7 @@ void Transfer::slotUpdateActions()
     m_paTimer->blockSignals(false);
     m_paDelay->blockSignals(false);
 
-    if (isVisible())
+    if (dlgIndividual)
         dlgIndividual->update();
 
     sDebugOut << endl;
@@ -274,20 +282,23 @@ void Transfer::updateAll()
     // destination
     setText(view->lv_filename, dest.fileName());
 
-    dlgIndividual->setCopying(src, dest);
-    dlgIndividual->setCanResume(canResume);
+    if(dlgIndividual)
+		{
+		dlgIndividual->setCopying(src, dest);
+    	dlgIndividual->setCanResume(canResume);
+		dlgIndividual->setTotalSize(totalSize);
+		dlgIndividual->setPercent(0);
+		dlgIndividual->setProcessedSize(0);
+	}
 
     if (totalSize != 0) {
-        //logMessage(i18n("Total size is %1 bytes").arg(totalSize));
+        //logMessage(i18n("Total size is %1 bytes").arg((double)totalSize));
         setText(view->lv_total, KIO::convertSize(totalSize));
     } else {
         //logMessage(i18n("Total size is unknown"));
         setText(view->lv_total, i18n("unknown"));
     }
 
-    dlgIndividual->setTotalSize(totalSize);
-    dlgIndividual->setPercent(0);
-    dlgIndividual->setProcessedSize(0);
 
     sDebugOut << endl;
 }
@@ -299,29 +310,47 @@ bool Transfer::updateStatus(int counter)
 
     QPixmap *pix = 0L;
     bool isTransfer = false;
+    static TransferStatus prevStatus;
 
-    if (status == ST_RUNNING) {
-        pix = view->animConn.at(counter);
-        isTransfer = true;
-    } else if (status == ST_TRYING) {
-        pix = view->animTry.at(counter);
-        isTransfer = true;
+    view->setUpdatesEnabled(false);
 
-    } else if (status == ST_STOPPED /*|| status==ST_PAUSED||status==ST_ABORTED */ ) {
-        if (mode == MD_QUEUED) {
-            pix = &view->pixQueued;
-        } else if (mode == MD_SCHEDULED) {
-            pix = &view->pixScheduled;
-        } else {
-            pix = &view->pixDelayed;
-        }
-    } else if (status == ST_FINISHED) {
-        pix = &view->pixFinished;
+    switch(status)
+        {
+        case ST_RUNNING:
+            pix = view->animConn.at(counter);
+        case ST_TRYING:
+            pix = view->animTry.at(counter);
+            isTransfer = true;
+            break;
+        case ST_STOPPED:
+            if(mode == MD_QUEUED)
+                pix = &view->pixQueued;
+            else if(mode == MD_SCHEDULED)
+                pix = &view->pixScheduled;
+            else
+                pix = &view->pixDelayed;
+            break;
+        case ST_FINISHED:
+            pix = &view->pixFinished;
     }
 
     setPixmap(view->lv_pixmap, *pix);
+    view->setUpdatesEnabled(true);
+
+    if(prevStatus!=status || status==ST_RUNNING || status==ST_TRYING)
+        {
+        QRect rect = view->header()->sectionRect(view->lv_pixmap);
+
+        int x = rect.x();
+        int y = view->itemRect(this).y();
+        int w = rect.width();
+        int h = rect.height();
+
+        view->QScrollView::updateContents(x,y,w,h);
+
+        prevStatus = status;
+    }
 
-    //sDebug<< "<<<<Leaving"<<endl;
     return isTransfer;
 }
 
@@ -356,6 +385,7 @@ void Transfer::slotResume()
 
     status = ST_TRYING;
     mode = MD_QUEUED;
+    logMessage(i18n("Attempt number %1").arg(retryCount));
 
     sDebug << "sending Resume to slave " << endl;
     m_pSlave->Op(Slave::RETR);
@@ -403,7 +433,8 @@ void Transfer::slotRequestRemove()
     sDebugIn << endl;
     m_paDelete->setEnabled(false);
     m_paPause->setEnabled(false);
-    dlgIndividual->close();
+    if(dlgIndividual)
+		dlgIndividual->close();
 
     if ( status != ST_FINISHED )
     {
@@ -503,7 +534,8 @@ void Transfer::slotFinished()
     slotProcessedSize(totalSize);
 
     slotSpeed(0);
-    dlgIndividual->enableOpenFile();
+    if(dlgIndividual)
+		dlgIndividual->enableOpenFile();
     emit statusChanged(this, OP_FINISHED);
     sDebugOut << endl;
 }
@@ -559,14 +591,15 @@ void Transfer::slotSpeed(unsigned long bytes_per_second)
         setText(view->lv_remaining, remainingTime.toString());
     }
 
-    dlgIndividual->setSpeed(speed, remainingTime);
+	if(dlgIndividual)
+    	dlgIndividual->setSpeed(speed, remainingTime);
 
     //sDebugOut<<endl;
 }
 
 
 
-void Transfer::slotTotalSize(unsigned long bytes)
+void Transfer::slotTotalSize(KIO::filesize_t bytes)
 {
 #ifdef _DEBUG
     sDebugIn<<" totalSize is = "<<totalSize << endl;
@@ -575,11 +608,14 @@ void Transfer::slotTotalSize(unsigned long bytes)
     if (totalSize == 0) {
         totalSize = bytes;
         if (totalSize != 0) {
-            logMessage(i18n("Total size is %1 bytes").arg(totalSize));
+            logMessage(i18n("Total size is %1 bytes").arg((double)totalSize,0,'f'));
             setText(view->lv_total, KIO::convertSize(totalSize));
-            dlgIndividual->setTotalSize(totalSize);
-            dlgIndividual->setPercent(0);
-            dlgIndividual->setProcessedSize(0);
+			if(dlgIndividual)
+				{
+				dlgIndividual->setTotalSize(totalSize);
+				dlgIndividual->setPercent(0);
+				dlgIndividual->setProcessedSize(0);
+			}
         }
     } else {
 
@@ -588,7 +624,7 @@ void Transfer::slotTotalSize(unsigned long bytes)
         assert(totalSize == bytes);
 #endif
         if (totalSize != bytes)
-            logMessage(i18n("The file size does not match!"));
+            logMessage(i18n("The file size does not match."));
         else
             logMessage(i18n("File Size checked"));
     }
@@ -600,7 +636,7 @@ void Transfer::slotTotalSize(unsigned long bytes)
 
 
 
-void Transfer::slotProcessedSize(unsigned long bytes)
+void Transfer::slotProcessedSize(KIO::filesize_t bytes)
 {
     //sDebug<< ">>>>Entering"<<endl;
 
@@ -617,12 +653,14 @@ void Transfer::slotProcessedSize(unsigned long bytes)
         totalSize = processedSize;
 
         setText(view->lv_total, KIO::convertSize(totalSize));
-        dlgIndividual->setTotalSize(totalSize);
+		if(dlgIndividual)
+        	dlgIndividual->setTotalSize(totalSize);
     }
     else {
         percent = (int) (((float) processedSize / (float) totalSize) * 100.0);
     }
-    dlgIndividual->setProcessedSize(processedSize);
+	if(dlgIndividual)
+    	dlgIndividual->setProcessedSize(processedSize);
 
     if (percent != old) {
         QString tmps;
@@ -634,7 +672,8 @@ void Transfer::slotProcessedSize(unsigned long bytes)
 
         setText(view->lv_progress, tmps);
 
-        dlgIndividual->setPercent(percent);
+		if(dlgIndividual)
+        	dlgIndividual->setPercent(percent);
     }
     //sDebug<< "<<<<Leaving"<<endl;
 }
@@ -646,6 +685,25 @@ void Transfer::showIndividual()
 {
     sDebugIn << endl;
 
+    //    create a DlgIndividual only if it hasn't been created yet
+    if(!dlgIndividual)
+    {
+        dlgIndividual = new DlgIndividual(this);
+        dlgIndividual->setLog(transferLog);
+        dlgIndividual->setCopying(src, dest);
+        dlgIndividual->setCanResume(canResume);
+        dlgIndividual->setTotalSize(totalSize);
+        dlgIndividual->setPercent(percent);
+        dlgIndividual->setProcessedSize(processedSize);
+    }
+
+    dlgIndividual->raise();
+
+
+    if (ksettings.b_iconifyIndividual) {
+        KWin::iconifyWindow( dlgIndividual->winId() );
+    }
+
     //      update the actions
     slotUpdateActions();
     //     then show the single dialog
@@ -656,13 +714,18 @@ void Transfer::showIndividual()
 }
 
 
-
 void Transfer::logMessage(const QString & message)
 {
     sDebugIn << message << endl;
 
     emit log(id, src.fileName(), message);
-    dlgIndividual->addLog(message);
+
+    QString tmps = "<font color=\"blue\">" + QTime::currentTime().toString() + "</font> : " + message;
+
+    transferLog.append(tmps + '\n');
+
+    if(dlgIndividual)
+        dlgIndividual->appendLog(tmps);
 
     sDebugOut << endl;
 }
@@ -691,8 +754,8 @@ bool Transfer::read(KSimpleConfig * config, int id)
     status = (TransferStatus) config->readNumEntry("Status", ST_RUNNING);
     startTime = config->readDateTimeEntry("ScheduledTime");
     canResume = config->readBoolEntry("CanResume", true);
-    totalSize = config->readNumEntry("TotalSize", 0);
-    processedSize = config->readNumEntry("ProcessedSize", 0);
+    totalSize = config->readUnsignedNum64Entry("TotalSize", 0);
+    processedSize = config->readUnsignedNum64Entry("ProcessedSize", 0);
 
     if (status != ST_FINISHED && totalSize != 0) {
         //TODO insert additional check
@@ -718,8 +781,8 @@ void Transfer::write(KSimpleConfig * config, int id)
     config->writeEntry("Mode", mode);
     config->writeEntry("Status", status);
     config->writeEntry("CanResume", canResume);
-    config->writeEntry("TotalSize", totalSize);
-    config->writeEntry("ProcessedSize", processedSize);
+    config->writeEntry("TotalSize", totalSize );
+    config->writeEntry("ProcessedSize", processedSize );
     config->writeEntry("ScheduledTime", startTime);
     sDebugOut << endl;
 }
@@ -746,42 +809,35 @@ void Transfer::slotExecPause()
     sDebugOut << endl;
 }
 
-void Transfer::slotExecAbort(const QString & _msg)
+void Transfer::slotExecError()
 {
-    mode = MD_DELAYED;
+    sDebugIn << endl;
+
     status = ST_STOPPED;
-    slotSpeed(0);               //need???????
-    QString tmps;
+    mode = MD_SCHEDULED;
+    startTime=QDateTime::currentDateTime().addSecs(ksettings.reconnectTime * 60);
+    emit statusChanged(this, OP_SCHEDULED);
 
-    tmps = "<code><font color=\"red\"> <strong>" + _msg + "</font> </strong></code><br/>";
-
-    logMessage(tmps);
-
-    if (ksettings.b_reconnectOnError) {
-        retryCount++;
-        if (retryCount == ksettings.reconnectRetries) { // no more retries
-            if (retryCount > ksettings.reconnectRetries)
-                ksettings.reconnectRetries = retryCount;
-            status = ST_STOPPED;
-            mode = MD_DELAYED;
-        } else {
-            status = ST_STOPPED;
-            mode = MD_SCHEDULED;
-            startTime = QDateTime::currentDateTime().addSecs(ksettings.reconnectTime * 60);
-            logMessage(i18n("Attempt number %1").arg(retryCount));
-        }
-    } else {                    // if reconnecting is not enabled - simply set to delayed
-        status = ST_STOPPED;
-        mode = MD_DELAYED;
-    }
-
-    emit statusChanged(this, OP_ABORTED);
+    sDebugOut << endl;
 }
 
+void Transfer::slotExecBroken()
+{
+    sDebugIn << endl;
+
+    status = ST_STOPPED;
+    mode = MD_QUEUED;
+    emit statusChanged(this, OP_QUEUED);
+
+    sDebugOut << endl;
+}
+
+
 void Transfer::slotExecRemove()
 {
     sDebugIn << endl;
 
+    m_pSlave->wait();
     emit statusChanged(this, OP_REMOVED);
     sDebugOut << endl;
 }
@@ -816,7 +872,7 @@ void Transfer::slotCanResume(bool _bCanResume)
         setText(view->lv_resume, i18n("No"));
     }
 
-    dlgIndividual->setCanResume(canResume);
+    //dlgIndividual->setCanResume(canResume);
 
     sDebugOut << endl;
 }
@@ -861,7 +917,7 @@ void Transfer::slotStartTime(const QDateTime & _startTime)
 /** return true if the dlgIndividual is Visible */
 bool Transfer::isVisible() const
 {
-    return dlgIndividual->isVisible();
+    return dlgIndividual ? dlgIndividual->isVisible() : false;
 }
 
 bool Transfer::keepDialogOpen() const
@@ -872,7 +928,20 @@ bool Transfer::keepDialogOpen() const
 void Transfer::maybeShow()
 {
     if ( ksettings.b_showIndividual && getStatus() != Transfer::ST_FINISHED )
-        dlgIndividual->show();
+    {
+        if(dlgIndividual)
+            dlgIndividual->show();
+    }
+}
+
+bool Transfer::retryOnError()
+{
+    return (ksettings.b_reconnectOnError && (retryCount <= ksettings.reconnectRetries));
+}
+
+bool Transfer::retryOnBroken()
+{
+    return (ksettings.b_reconnectOnBroken && (retryCount <= ksettings.reconnectRetries));
 }
 
 #include "transfer.moc"
