@@ -37,22 +37,28 @@
 #include "kopetecontactlist.h"
 #include "kopetegroup.h"
 #include "kopetemetacontact.h"
-
+#include "kopetepassword.h"
+#include "kopeteuiglobal.h"
+#include "kopeteglobal.h"
 
 #include "sha1.h"
 
 
+#if !defined NDEBUG
+#include "msndebugrawcmddlg.h"
+#include <kglobal.h>
+#endif
+
 MSNAccount::MSNAccount( MSNProtocol *parent, const QString& AccountID, const char *name )
-: KopeteAccount ( parent, AccountID, name )
+: Kopete::PasswordedAccount ( parent, AccountID, 0, name )
 {
 	m_notifySocket = 0L;
 	m_connectstatus = MSNProtocol::protocol()->NLN;
 	m_addWizard_metaContact = 0L;
-	m_badpassword = false;
 
 	// Init the myself contact
 	// FIXME: I think we should add a global self metaContact ( Olivier )
-	setMyself( new MSNContact( this, accountId(), accountId(), 0L ) );
+	setMyself( new MSNContact( this, accountId(), 0L ) );
 	//myself()->setOnlineStatus( MSNProtocol::protocol()->FLN );
 
 	QObject::connect( KopeteContactList::contactList(), SIGNAL( groupRenamed( KopeteGroup *, const QString & ) ),
@@ -60,15 +66,17 @@ MSNAccount::MSNAccount( MSNProtocol *parent, const QString& AccountID, const cha
 	QObject::connect( KopeteContactList::contactList(), SIGNAL( groupRemoved( KopeteGroup * ) ),
 		SLOT( slotKopeteGroupRemoved( KopeteGroup * ) ) );
 
-	m_openInboxAction = new KAction ( i18n( "Open Inbo&x..." ), "mail_generic", 0, this, SLOT( slotOpenInbox() ), this, "m_openInboxAction" );
-	m_openInboxAction->setEnabled( false );
+	m_openInboxAction = new KAction( i18n( "Open Inbo&x..." ), "mail_generic", 0, this, SLOT( slotOpenInbox() ), this, "m_openInboxAction" );
+	m_changeDNAction = new KAction( i18n( "&Change Display Name..." ), QString::null, 0, this, SLOT( slotChangePublicName() ), this, "renameAction" );
+	m_startChatAction = new KAction( i18n( "&Start Chat..." ), "mail_generic", 0, this, SLOT( slotStartChat() ), this, "startChatAction" );
+
 }
 
 void MSNAccount::loaded()
 {
 	QString publicName = pluginData( protocol(), QString::fromLatin1( "displayName" ) );
 	if ( !publicName.isNull() )
-		static_cast<MSNContact *>( myself() )->setDisplayName( publicName );
+		myself()->setProperty( Kopete::Global::Properties::self()->nickName() , publicName );
 	m_blockList   = QStringList::split( ',', pluginData( protocol(), QString::fromLatin1( "blockList" ) ) );
 	m_allowList   = QStringList::split( ',', pluginData( protocol(), QString::fromLatin1( "allowList" ) ) );
 	m_reverseList = QStringList::split( ',', pluginData( protocol(), QString::fromLatin1( "reverseList" ) ) );
@@ -88,6 +96,26 @@ void MSNAccount::loaded()
 	}
 }
 
+QString MSNAccount::serverName()
+{
+	//FIXME: After need for backwards compatability is removed, just return serverName
+	QString sName = pluginData( protocol(), QString::fromLatin1( "serverName" ) );
+	if( !sName.isEmpty() )
+		return sName;
+	else
+		return QString::fromLatin1("messenger.hotmail.com");
+}
+
+uint MSNAccount::serverPort()
+{
+	//FIXME: After need for backwards compatability is removed, just return serverPort
+	QString serverPort = pluginData( protocol(), QString::fromLatin1( "serverPort" ) );
+	if( !serverPort.isEmpty() )
+		return serverPort.toUInt();
+	else
+		return 1863;
+}
+
 void MSNAccount::setAway( bool away, const QString & awayReason )
 {
 	m_awayReason = awayReason;
@@ -97,38 +125,58 @@ void MSNAccount::setAway( bool away, const QString & awayReason )
 		setOnlineStatus( MSNProtocol::protocol()->NLN );
 }
 
-void MSNAccount::connect()
+void MSNAccount::connectWithPassword( const QString &passwd )
 {
 	if ( isConnected() )
 	{
-		kdDebug( 14140 ) << "MSNAccount::connect: Ignoring Connect request "
+		kdDebug( 14140 ) << k_funcinfo <<"Ignoring Connect request "
 			<< "(Already Connected)" << endl;
 		return;
 	}
 
 	if ( m_notifySocket )
 	{
-		kdDebug( 14140 ) << "MSNAccount::connect: Ignoring Connect request (Already connecting)"  << endl;
+		kdDebug( 14140 ) << k_funcinfo <<"Ignoring Connect request (Already connecting)"  << endl;
 		return;
 	}
 
+	m_password = passwd; 
 
-	QString passwd = password( m_badpassword );
-	m_badpassword = false;
-	if ( passwd.isNull() )
+	if ( m_password.isNull() )
 	{
-		kdDebug( 14140 ) << "MSNAccount::connect: Abort connection (null password)"  << endl;
+		kdDebug( 14140 ) << k_funcinfo <<"Abort connection (null password)"  << endl;
 		return;
 	}
 
+
 	if ( contacts().count() <= 1 )
 	{
 		// Maybe the contactlist.xml has been removed, and the serial number not updated
 		// ( the 1 is for the myself contact )
 		setPluginData( protocol(), "serial", "0" );
 	}
+	
+	m_openInboxAction->setEnabled( false );
+	
+	createNotificationServer(serverName(), serverPort());
+}
 
-	m_notifySocket = new MSNNotifySocket( this, accountId() , passwd);
+void MSNAccount::createNotificationServer( const QString &host, uint port )
+{
+	if(m_notifySocket) //we are switching from one to another notifysocket.
+	{
+		//remove every slots to that socket, so we won't delete receive signals
+		// from the old socket thinking they are from the new one
+		QObject::disconnect( m_notifySocket , 0, this, 0 ); 
+		m_notifySocket->deleteLater(); //be sure it will be deleted
+		m_notifySocket=0L; 
+	}
+
+
+	myself()->setOnlineStatus( MSNProtocol::protocol()->CNT );
+
+
+	m_notifySocket = new MSNNotifySocket( this, accountId() , m_password);
 
 	QObject::connect( m_notifySocket, SIGNAL( groupAdded( const QString&, uint ) ),
 		SLOT( slotGroupAdded( const QString&, uint ) ) );
@@ -154,17 +202,17 @@ void MSNAccount::connect()
 		SLOT( slotCreateChat( const QString&, const QString&, const QString&, const QString&, const QString& ) ) );
 	QObject::connect( m_notifySocket, SIGNAL( startChat( const QString&, const QString& ) ),
 		SLOT( slotCreateChat( const QString&, const QString& ) ) );
-	QObject::connect( m_notifySocket, SIGNAL( socketClosed( int ) ),
-		SLOT( slotNotifySocketClosed( int ) ) );
+	QObject::connect( m_notifySocket, SIGNAL( socketClosed() ),
+		SLOT( slotNotifySocketClosed() ) );
 	QObject::connect( m_notifySocket, SIGNAL( newContactList() ),
 		SLOT( slotNewContactList() ) );
+	QObject::connect( m_notifySocket, SIGNAL( receivedNotificationServer(const QString&, uint )  ), 
+		SLOT(createNotificationServer(const QString&, uint ) ) );
 	QObject::connect( m_notifySocket, SIGNAL( hotmailSeted( bool ) ),
 		m_openInboxAction, SLOT( setEnabled( bool ) ) );
-
+	
 	m_notifySocket->setStatus( m_connectstatus );
-	m_notifySocket->connect();
-	myself()->setOnlineStatus( MSNProtocol::protocol()->CNT );
-	m_openInboxAction->setEnabled( false );
+	m_notifySocket->connect(host, port);
 }
 
 void MSNAccount::disconnect()
@@ -177,12 +225,20 @@ KActionMenu * MSNAccount::actionMenu()
 {
 	KActionMenu *m_actionMenu = new KActionMenu( accountId(), myself()->onlineStatus().iconFor( this ),  this );
 	m_actionMenu->popupMenu()->insertTitle( myself()->onlineStatus().iconFor( myself() ), i18n( "%2 <%1>" ).
-#if QT_VERSION < 0x030200
-		arg( accountId() ).arg( myself()->displayName() )
-#else
-		arg( accountId(), myself()->displayName() )
-#endif
-	);
+		arg( accountId(), myself()->property( Kopete::Global::Properties::self()->nickName()).value().toString() ) );
+
+	if ( isConnected() )
+	{
+		m_openInboxAction->setEnabled( true );
+		m_startChatAction->setEnabled( true );
+		m_changeDNAction->setEnabled( true );
+	}
+	else
+	{
+		m_openInboxAction->setEnabled( false );
+		m_startChatAction->setEnabled( false );
+		m_changeDNAction->setEnabled( false );
+	}
 
 	m_actionMenu->insert( new KAction ( i18n( "Set O&nline" ),        MSNProtocol::protocol()->NLN.iconFor( this ), 0,
 		this, SLOT( slotGoOnline() ), m_actionMenu, "actionMSNConnect" ) );
@@ -202,13 +258,22 @@ KActionMenu * MSNAccount::actionMenu()
 		this, SLOT( slotGoOffline() ), m_actionMenu, "actionMSNConnect" ) );
 
 	m_actionMenu->popupMenu()->insertSeparator();
-	m_actionMenu->insert( new KAction ( i18n( "&Change Display Name..." ), QString::null, 0,
-		this, SLOT( slotChangePublicName() ), m_actionMenu, "renameAction" ) );
-	m_actionMenu->insert( new KAction ( i18n( "&Start Chat..." ), "mail_generic", 0,
-		this, SLOT( slotStartChat() ), m_actionMenu, "startChatAction" ) );
+
+	m_actionMenu->insert( m_changeDNAction );
+	m_actionMenu->insert( m_startChatAction );
+
 	m_actionMenu->popupMenu()->insertSeparator();
+
 	m_actionMenu->insert( m_openInboxAction );
 
+#if !defined NDEBUG
+	KActionMenu *debugMenu = new KActionMenu( "Debug", m_actionMenu );
+	debugMenu->insert( new KAction( i18n( "Send Raw C&ommand..." ), 0,
+		this, SLOT( slotDebugRawCommand() ), debugMenu, "m_debugRawCommand" ) );
+	m_actionMenu->popupMenu()->insertSeparator();
+	m_actionMenu->insert( debugMenu );
+#endif
+
 	return m_actionMenu;
 }
 
@@ -219,12 +284,8 @@ MSNNotifySocket *MSNAccount::notifySocket()
 
 void MSNAccount::slotGoOnline()
 {
-	// m_connectstatus = NLN;
 	m_awayReason = QString::null;
-	if ( !isConnected() )
-		connect();
-	else
-		setOnlineStatus( MSNProtocol::protocol()->NLN );
+	setOnlineStatus( MSNProtocol::protocol()->NLN );
 }
 
 void MSNAccount::slotGoOffline()
@@ -284,12 +345,6 @@ void MSNAccount::setOnlineStatus( const KopeteOnlineStatus &status )
 
 void MSNAccount::slotStartChat()
 {
-	if ( !isConnected() )
-	{
-		KMessageBox::queuedMessageBox( 0L, KMessageBox::Error, i18n( "<qt>Please go online before you start a chat.</qt>" ),
-			i18n( "MSN Plugin" ), KMessageBox::Notify );
-		return;
-	}
 
 	bool ok;
 	QString handle = KInputDialog::getText( i18n( "Start Chat - MSN Plugin" ),
@@ -305,25 +360,41 @@ void MSNAccount::slotStartChat()
 		}
 		else
 		{
-			KMessageBox::queuedMessageBox( 0L, KMessageBox::Sorry,
+			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Sorry,
 				i18n( "<qt>You must enter a valid email address.</qt>" ), i18n( "MSN Plugin" ) );
 		}
 	}
 }
 
+void MSNAccount::slotDebugRawCommand()
+{
+#if !defined NDEBUG
+	if ( !isConnected() )
+		return;
+
+	MSNDebugRawCmdDlg *dlg = new MSNDebugRawCmdDlg( 0L );
+	int result = dlg->exec();
+	if ( result == QDialog::Accepted && m_notifySocket )
+	{
+		m_notifySocket->sendCommand( dlg->command(), dlg->params(),
+					dlg->addId(), dlg->msg().replace( "\n", "\r\n" ).utf8() );
+	}
+	delete dlg;
+#endif
+}
 
 void MSNAccount::slotChangePublicName()
 {
 	bool ok;
 	QString name = KInputDialog::getText( i18n( "Change Display Name - MSN Plugin" ),
 		i18n( "Enter the new display name by which you want to be visible to your friends on MSN:" ),
-		myself()->displayName(), &ok );
+		myself()->property( Kopete::Global::Properties::self()->nickName()).value().toString(), &ok );
 
 	if ( ok )
 	{
 		if ( name.length() > 387 )
 		{
-			KMessageBox::error( 0L,
+			KMessageBox::error( Kopete::UI::Global::mainWidget(),
 				i18n( "<qt>The display name you entered is too long. Please use a shorter name.\n"
 					"Your display name has <b>not</b> been changed.</qt>" ),
 				i18n( "Change Display Name - MSN Plugin" ) );
@@ -422,53 +493,42 @@ void MSNAccount::slotNotifySocketStatusChanged( MSNSocket::OnlineStatus status )
 	// static_cast<MSNProtocol *>( protocol() )->slotNotifySocketStatusChanged( status );
 }
 
-void MSNAccount::slotNotifySocketClosed( int  state  )
+void MSNAccount::slotNotifySocketClosed()
 {
 	kdDebug( 14140 ) << k_funcinfo << endl;
 
-	m_badpassword = m_notifySocket->badPassword();
+	password().setWrong( m_notifySocket->badPassword() );
 	m_notifySocket->deleteLater();
 	m_notifySocket = 0l;
 	myself()->setOnlineStatus( MSNProtocol::protocol()->FLN );
-	if ( m_badpassword )
+	if ( password().isWrong() )
 		connect();
+
+#if 0
 	else if ( state == 0x10 ) // connection died unexpectedly
 	{
-		KMessageBox::queuedMessageBox( 0, KMessageBox::Error , i18n( "The connection with the MSN server was lost unexpectedly.\n"
+		KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Error , i18n( "The connection with the MSN server was lost unexpectedly.\n"
 			"If you cannot reconnect now, the server might be down. In that case, please try again later." ),
 			i18n( "Connection Lost - MSN Plugin" ), KMessageBox::Notify );
 	}
+#endif
 
 	// kdDebug( 14140 ) << "MSNAccount::slotNotifySocketClosed - done" << endl;
 }
 
 void MSNAccount::slotStatusChanged( const KopeteOnlineStatus &status )
 {
-	kdDebug( 14140 ) << "MSNAccount::slotStatusChanged: " << status.internalStatus() <<  endl;
+//	kdDebug( 14140 ) << k_funcinfo  << status.internalStatus() <<  endl;
 	myself()->setOnlineStatus( status );
 }
 
 void MSNAccount::slotPublicNameChanged( const QString& publicName )
 {
-	if ( publicName != myself()->displayName() )
+	QString oldNick=myself()->property( Kopete::Global::Properties::self()->nickName()).value().toString() ;
+	if ( publicName != oldNick )
 	{
-		// if ( m_publicNameSyncMode & SyncFromServer )
-		// {
-			static_cast<MSNContact *>( myself() )->setDisplayName( publicName );
-			setPluginData( protocol(), QString::fromLatin1( "displayName" ), publicName );
-
-/*
-			m_publicNameSyncMode = SyncBoth;
-		}
-		else
-		{
-			// Check if name differs, and schedule sync if needed
-			if ( m_publicNameSyncMode & SyncToServer )
-				m_publicNameSyncNeeded = true;
-			else
-				m_publicNameSyncNeeded = false;
-		}
-*/
+		myself()->setProperty( Kopete::Global::Properties::self()->nickName(), publicName );
+		setPluginData( protocol(), QString::fromLatin1( "displayName" ), publicName );
 	}
 }
 
@@ -482,20 +542,6 @@ void MSNAccount::setPublicName( const QString &publicName )
 
 void MSNAccount::slotGroupAdded( const QString& groupName, uint groupNumber )
 {
-	// We have pending groups that we need add a contact to
-	if ( tmp_addToNewGroup.contains(groupName) )
-	{
-		QStringList list=tmp_addToNewGroup[groupName];
-		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
-		{
-			QString contactId =*it;
-				kdDebug( 14140 ) << k_funcinfo << "Adding to new group: " << contactId <<  endl;
-				notifySocket()->addContact( contactId, contacts()[ contactId ] ? contacts()[ contactId ]->displayName()
-					: contactId, groupNumber, MSNProtocol::FL );
-			}
-		tmp_addToNewGroup.remove(groupName);
-	}
-
 	if ( m_groupList.contains( groupNumber ) )
 	{
 		// Group can already be in the list since the idle timer does a 'List Groups'
@@ -503,43 +549,56 @@ void MSNAccount::slotGroupAdded( const QString& groupName, uint groupNumber )
 		// kdDebug( 14140 ) << k_funcinfo << "Group " << groupName << " already in list, skipped." << endl;
 		return;
 	}
-
-	// Find the appropriate KopeteGroup, or create one
+	
+	//--------- Find the appropriate KopeteGroup, or create one ---------//
 	QPtrList<KopeteGroup> groupList = KopeteContactList::contactList()->groups();
 	KopeteGroup *fallBack = 0L;
-	for ( KopeteGroup *g = groupList.first(); g; g = groupList.next() )
+	
+	//check if we have one in the old group list. if yes, update the id translate map.
+	for(QMap<unsigned int, KopeteGroup*>::Iterator it=m_oldGroupList.begin() ; it != m_oldGroupList.end() ; ++it )
 	{
-		// FIXME:  a different mapping for each accounts will be welcome!
-		if ( !g->pluginData( protocol(), accountId() +  " id" ).isEmpty() )
-		{
-			if ( g->pluginData( protocol(), accountId() + " id" ).toUInt() == groupNumber )
+		KopeteGroup *g=it.data();
+		if (g && g->pluginData( protocol(), accountId() + " displayName" ) == groupName  &&
+				g->pluginData( protocol(), accountId() + " id" ).isEmpty() )
+		{ //it has the same name! we got it.    (and it is not yet an msn group)
+			fallBack=g;
+			/*if ( g->displayName() != groupName )
 			{
-				m_groupList.insert( groupNumber, g );
-				QString oldGroupName;
-				if ( g->pluginData( protocol(), accountId() + " displayName" ) != groupName )
-				{
-					// The displayName of the group has been modified by another client
-					slotGroupRenamed( groupName, groupNumber );
-				}
-				else if ( g->displayName() != groupName )
-				{
-					// The displayName was changed in Kopete while we were offline
-					// FIXME: update the server right now
-				}
-				return;
-			}
+				// The displayName was changed in Kopete while we were offline
+				// FIXME: update the server right now
+			}*/
+			break;
 		}
-		else
+	}
+	
+	if(!fallBack)
+	{
+		//it's certenly a new group !  search if one already exist with the same displayname.
+		for ( KopeteGroup *g = groupList.first(); g; g = groupList.next() )
 		{
-			// If we found a group with the same displayName but no plugin data
-			// use that instead. This group is only used if no exact match is
-			// found in the list.  Note that only the group #0 can be the top-level
-			// FIXME: When adding a group in Kopete we already need to inform the
-			//        plugins about the KopeteGroup *, so they know which to use
-			//        and this code path can be removed ( or kept solely for people
-			//        who migrate from older versions ) - Martijn
-			if ( g->displayName() == groupName && (groupNumber==0 || g->type()==KopeteGroup::Normal)  )
+			/*   --This has been replaced by the loop right before.
+			if ( !g->pluginData( protocol(), accountId() +  " id" ).isEmpty() )
+			{
+				if ( g->pluginData( protocol(), accountId() + " id" ).toUInt() == groupNumber )
+				{
+					m_groupList.insert( groupNumber, g );
+					QString oldGroupName;
+					if ( g->pluginData( protocol(), accountId() + " displayName" ) != groupName )
+					{
+						// The displayName of the group has been modified by another client
+						slotGroupRenamed( groupName, groupNumber );
+					}
+					return;
+				}
+			}
+			else {*/
+			if ( g->displayName() == groupName && (groupNumber==0 || g->type()==KopeteGroup::Normal)  &&
+					g->pluginData( protocol(), accountId() + " id" ).isEmpty()  )
+			{
 				fallBack = g;
+				kdDebug( 14140 ) << k_funcinfo << "We didn't found the group " << groupName <<" in the old MSN group.  But kopete has already one with the same name." <<  endl;
+				break;
+			}
 		}
 	}
 
@@ -548,21 +607,38 @@ void MSNAccount::slotGroupAdded( const QString& groupName, uint groupNumber )
 		if( groupNumber==0  )
 		{	// The group #0 is an unremovable group. his default name is "~" ,
 			// but the official client rename it i18n("others contact") at the first
-			// connection.   
-			// In many case, the users don't use that group as a real group, or just as 
-			// a group to put all contact that are not sorted.  
+			// connection.
+			// In many case, the users don't use that group as a real group, or just as
+			// a group to put all contact that are not sorted.
 			fallBack = KopeteGroup::topLevel();
 		}
 		else
 		{
-		fallBack = new KopeteGroup( groupName );
-		KopeteContactList::contactList()->addGroup( fallBack );
-	}
+			fallBack = new KopeteGroup( groupName );
+			KopeteContactList::contactList()->addGroup( fallBack );
+			kdDebug( 14140 ) << k_funcinfo << "We didn't found the group " << groupName <<" So we're creating a new one." <<  endl;
+
+		}
 	}
 
 	fallBack->setPluginData( protocol(), accountId() + " id", QString::number( groupNumber ) );
 	fallBack->setPluginData( protocol(), accountId() + " displayName", groupName );
 	m_groupList.insert( groupNumber, fallBack );
+	
+	// We have pending groups that we need add a contact to
+	if ( tmp_addToNewGroup.contains(groupName) )
+	{
+		QStringList list=tmp_addToNewGroup[groupName];
+		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
+		{
+			QString contactId =*it;
+			kdDebug( 14140 ) << k_funcinfo << "Adding to new group: " << contactId <<  endl;
+			notifySocket()->addContact( contactId, contacts()[ contactId ] ? 
+					contacts()[contactId]->property( Kopete::Global::Properties::self()->nickName()).value().toString()
+					: contactId, groupNumber, MSNProtocol::FL );
+		}
+		tmp_addToNewGroup.remove(groupName);
+	}
 }
 
 void MSNAccount::slotGroupRenamed( const QString& groupName, uint groupNumber )
@@ -592,7 +668,7 @@ void MSNAccount::slotGroupRemoved( uint group )
 
 void MSNAccount::addGroup( const QString &groupName, const QString& contactToAdd )
 {
-	if ( !contactToAdd.isNull() )
+	if ( !contactToAdd.isNull()  )
 	{
 		if( tmp_addToNewGroup.contains(groupName) )
 		{
@@ -611,6 +687,7 @@ void MSNAccount::addGroup( const QString &groupName, const QString& contactToAdd
 
 	if ( m_notifySocket )
 		m_notifySocket->addGroup( groupName );
+
 }
 
 void MSNAccount::slotKopeteGroupRenamed( KopeteGroup *g )
@@ -618,10 +695,10 @@ void MSNAccount::slotKopeteGroupRenamed( KopeteGroup *g )
 	if ( notifySocket() && g->type() == KopeteGroup::Normal )
 	{
 		if ( !g->pluginData( protocol(), accountId() + " id" ).isEmpty() &&
-			g->displayName() != g->pluginData( protocol(), accountId() + " displayName" ) && 
+			g->displayName() != g->pluginData( protocol(), accountId() + " displayName" ) &&
 			m_groupList.contains( g->pluginData( protocol(), accountId() + " id" ).toUInt() ) )
 		{
-				notifySocket()->renameGroup( g->displayName(), g->pluginData( protocol(), accountId() + " id" ).toUInt() );
+			notifySocket()->renameGroup( g->displayName(), g->pluginData( protocol(), accountId() + " id" ).toUInt() );
 		}
 	}
 }
@@ -665,11 +742,30 @@ void MSNAccount::slotKopeteGroupRemoved( KopeteGroup *g )
 			}*/
 			m_notifySocket->removeGroup( groupNumber );
 		}
+		//this is also done later, but he have to do it now!
+		// (in slotGroupRemoved)
+		m_groupList.remove(groupNumber);
+	}
+	
+	//remove it from the old list
+	for(QMap<unsigned int, KopeteGroup*>::Iterator it=m_oldGroupList.begin() ; it != m_oldGroupList.end() ; ++it )
+	{
+		if(g==it.data())
+		{
+			m_oldGroupList.remove(it);
+			break;
+		}
 	}
 }
 
 void MSNAccount::slotNewContactList()
 {
+		m_oldGroupList=m_groupList;
+		for(QMap<unsigned int, KopeteGroup*>::Iterator it=m_oldGroupList.begin() ; it != m_oldGroupList.end() ; ++it )
+		{	//they are about to be changed 
+			it.data()->setPluginData( protocol(), accountId() + " id", QString::null );
+		}
+
 		m_allowList.clear();
 		m_blockList.clear();
 		m_reverseList.clear();
@@ -698,94 +794,99 @@ void MSNAccount::slotContactListed( const QString& handle, const QString& public
 	// On empty lists handle might be empty, ignore that
 	if ( handle.isEmpty() )
 		return;
+		
+	MSNContact *c = static_cast<MSNContact *>( contacts()[ handle ] );
 
 	if ( lists & 1 )	// FL
 	{
 		QStringList contactGroups = QStringList::split( ",", group, false );
-		KopeteMetaContact *metaContact = KopeteContactList::contactList()->findContact( protocol()->pluginId(), accountId(), handle );
-		if ( metaContact )
+		if ( c )
 		{
+			if( !c->metaContact() )
+			{
+				kdWarning( 14140 ) << k_funcinfo << "the contact " << c->contactId() << " has no meta contact" <<endl;
+				KopeteMetaContact *metaContact = new KopeteMetaContact();
+
+				c->setMetaContact(metaContact);
+				KopeteContactList::contactList()->addMetaContact( metaContact );
+			}
+
 			// Contact exists, update data.
 			// Merging difference between server contact list and KopeteContact's contact list into MetaContact's contact-list
-			MSNContact *c = static_cast<MSNContact *>( metaContact->findContact( protocol()->pluginId(), accountId(), handle ) );
 			c->setOnlineStatus( MSNProtocol::protocol()->FLN );
-			c->setDisplayName( publicName );
+			c->setProperty( Kopete::Global::Properties::self()->nickName() ,  publicName );
 
-			const QMap<uint, KopeteGroup *>serverGroups = c->serverGroups();
+			const QMap<uint, KopeteGroup *> oldServerGroups = c->serverGroups();
+			c->clearServerGroups();  
 			for ( QStringList::ConstIterator it = contactGroups.begin(); it != contactGroups.end(); ++it )
 			{
-				uint serverGroup = ( *it ).toUInt();
-				if ( !serverGroups.contains( serverGroup ) )
+				uint newServerGroupID =  ( *it ).toUInt();
+				KopeteGroup *newServerGroup=m_groupList[ newServerGroupID ];
+				c->contactAddedToGroup( newServerGroupID, newServerGroup );
+				if( !c->metaContact()->groups().contains(newServerGroup) )
 				{
 					// The contact has been added in a group by another client
-					c->contactAddedToGroup( serverGroup, m_groupList[ serverGroup ] );
-					c->setDontSync( true ); // prevent the moving of the metacontact change the server
-					metaContact->addToGroup( m_groupList[ serverGroup ] );
+					c->metaContact()->addToGroup( newServerGroup , KopeteMetaContact::DontSyncGroups  );
 				}
 			}
 
-			for ( QMap<uint, KopeteGroup *>::ConstIterator it = serverGroups.begin(); it != serverGroups.end(); ++it )
+			for ( QMap<uint, KopeteGroup *>::ConstIterator it = oldServerGroups.begin(); it != oldServerGroups.end(); ++it )
 			{
-				if ( !contactGroups.contains( QString::number( it.key() ) ) )
+				KopeteGroup *old_group=m_oldGroupList[it.key()];
+				if(old_group)
 				{
-					// The contact has been removed from a group by another client
-					c->contactRemovedFromGroup( it.key() );
-					c->setDontSync( true ); // prevent the moving of the metacontact change the server
+					QString oldnewID=old_group->pluginData(protocol() , accountId() +" id");
+					if ( !oldnewID.isEmpty() && contactGroups.contains( oldnewID ) )
+						continue; //ok, it's correctn no need to do anything.
 					
-					KopeteGroup *old_group=m_groupList.contains( it.key() ) ? m_groupList[it.key()] : it.data();
-					if(!old_group)
-					{	//the group is not anymore on the msn server.
-						QPtrList<KopeteGroup> mc_groups = c->metaContact()->groups();
-						if(mc_groups.count() > 1) 
-							for(KopeteGroup *g_it=mc_groups.first() ; g_it ; g_it=mc_groups.next() )
-						{
-							QString Gid=g_it->pluginData(protocol() , accountId() +" id");
-							if(Gid.isEmpty())
-							{
-								//maybe it's that one, it has not id
-								old_group=g_it;
-								continue; //but continue searching, another is maybe better
-							}
-							else if( Gid.toUInt()==it.key() )
-							{
-								old_group=g_it;
-								break;
-							}
-						}
-						
-					}
-					metaContact->removeFromGroup( old_group );
+					c->metaContact()->removeFromGroup( old_group , KopeteMetaContact::DontSyncGroups );
 				}
 			}
 
 			// Update server if the contact has been moved to another group while MSN was offline
-			c->setDontSync( false ); // now, allow to change the server
 			c->syncGroups();
 		}
 		else
 		{
-			metaContact = new KopeteMetaContact();
+			KopeteMetaContact *metaContact = new KopeteMetaContact();
 
-			MSNContact *msnContact = new MSNContact( this, handle, publicName, metaContact );
-			msnContact->setOnlineStatus( MSNProtocol::protocol()->FLN );
+			c = new MSNContact( this, handle, metaContact );
+			c->setOnlineStatus( MSNProtocol::protocol()->FLN );
+			c->setProperty( Kopete::Global::Properties::self()->nickName() , publicName );
 
 			for ( QStringList::Iterator it = contactGroups.begin();
 				it != contactGroups.end(); ++it )
 			{
 				uint groupNumber = ( *it ).toUInt();
-				msnContact->contactAddedToGroup( groupNumber, m_groupList[ groupNumber ] );
+				c->contactAddedToGroup( groupNumber, m_groupList[ groupNumber ] );
 				metaContact->addToGroup( m_groupList[ groupNumber ] );
 			}
 			KopeteContactList::contactList()->addMetaContact( metaContact );
 		}
 	}
+	else //the contact is _not_ in the FL, it has been removed
+	{
+		if(c)
+		{
+			c->setOnlineStatus( static_cast<MSNProtocol*>(protocol())->UNK );
+			c->clearServerGroups();
+			//TODO: display a message and suggest to remove the contact.
+			//  but i fear a simple messageBox  QuestionYesNo here gives a nice crash.
+			   //delete ct;
+		}
+	}
 	if ( lists & 2 )
 		slotContactAdded( handle, publicName, "AL", 0 );
+	else if(c)
+		c->setAllowed(false);
 	if ( lists & 4 )
 		slotContactAdded( handle, publicName, "BL", 0 );
+	else if(c)
+		c->setBlocked(false);
 	if ( lists & 8 )
 		slotContactAdded( handle, publicName, "RL", 0 );
-
+	else if(c)
+		c->setReversed(false);
 }
 
 void MSNAccount::slotContactAdded( const QString& handle, const QString& publicName,
@@ -796,6 +897,7 @@ void MSNAccount::slotContactAdded( const QString& handle, const QString& publicN
 		bool new_contact = false;
 		if ( !contacts()[ handle ] )
 		{
+			// FIXME: findContact() only works if the account id is an existing account - see KopeteAccountManager::findAccount() for the reason
 			KopeteMetaContact *m = KopeteContactList::contactList()->findContact( protocol()->pluginId(), QString::null, handle );
 			if ( m )
 			{
@@ -813,8 +915,9 @@ void MSNAccount::slotContactAdded( const QString& handle, const QString& publicN
 				else
 					m = new KopeteMetaContact();
 
-				MSNContact *c = new MSNContact( this, handle, publicName, m );
+				MSNContact *c = new MSNContact( this, handle, m );
 				c->contactAddedToGroup( group, m_groupList[ group ] );
+				c->setProperty( Kopete::Global::Properties::self()->nickName() , publicName);
 
 				if ( !m_addWizard_metaContact )
 				{
@@ -967,7 +1070,7 @@ void MSNAccount::slotCreateChat( const QString& ID, const QString& address, cons
 		return;
 	}
 
-	kdDebug( 14140 ) << "MSNAccount::slotCreateChat: Creating chat for " << handle << endl;
+//	kdDebug( 14140 ) << k_funcinfo <<"Creating chat for " << handle << endl;
 
 	if ( !contacts()[ handle ] )
 		addContact( handle, publicName, 0L, KopeteAccount::DontChangeKABC, QString::null, true );
@@ -985,7 +1088,7 @@ void MSNAccount::slotCreateChat( const QString& ID, const QString& address, cons
 			chatmembers.append(c);
 			manager = new MSNMessageManager( protocol(), myself(), chatmembers  );
 		}
-	
+
 		manager->createChat( handle, address, auth, ID );
 
 		KGlobal::config()->setGroup( "MSN" );
@@ -993,7 +1096,7 @@ void MSNAccount::slotCreateChat( const QString& ID, const QString& address, cons
 		if ( !ID.isEmpty() && notifyNewChat )
 		{
 			// this temporary message should open the window if they not exist
-			QString body = i18n( "%1 has started a chat with you" ).arg( c->displayName() );
+			QString body = i18n( "%1 has started a chat with you" ).arg( c->metaContact()->displayName() );
 			KopeteMessage tmpMsg = KopeteMessage( c, manager->members(), body, KopeteMessage::Internal, KopeteMessage::PlainText );
 			manager->appendMessage( tmpMsg );
 		}
@@ -1069,9 +1172,9 @@ bool MSNAccount::addContactToMetaContact( const QString &contactId, const QStrin
 				{
 					int Gid=group->pluginData( protocol(), accountId() + " id" ).toUInt();
 					if(!m_groupList.contains(Gid))
-					{ // ohoh!   something is corrupted on the contactlist.xml  
+					{ // ohoh!   something is corrupted on the contactlist.xml
 					  // anyway, we never should add a contact to an unexisting group on the server.
-					  
+
 						//repair the problem
 						group->setPluginData( protocol() , accountId() + " id" , QString::null);
 						group->setPluginData( protocol() , accountId() + " displayName" , QString::null);
@@ -1080,14 +1183,14 @@ bool MSNAccount::addContactToMetaContact( const QString &contactId, const QStrin
 					}
 					else
 					{
-					// Add the contact to the group on the server
+						// Add the contact to the group on the server
 						m_notifySocket->addContact( contactId, displayName, Gid, MSNProtocol::FL );
-					added = true;
-				}
+						added = true;
+					}
 				}
 				if(!added)
 				{
-					if ( !group->displayName().isEmpty() && group->type() == KopeteGroup::Normal ) 
+					if ( !group->displayName().isEmpty() && group->type() == KopeteGroup::Normal )
 					{  // not the top-level
 						// Create the group and add the contact
 						// FIXME: if for a reason or another the group can't be added, the contact will not be added.
@@ -1109,7 +1212,7 @@ bool MSNAccount::addContactToMetaContact( const QString &contactId, const QStrin
 		{
 			// This is a temporary contact. ( a person who messaged us but is not on our conntact list.
 			// We don't want to create it on the server.Just create the local contact object and add it
-			MSNContact *newContact = new MSNContact( this, contactId, contactId, metaContact );
+			MSNContact *newContact = new MSNContact( this, contactId, metaContact );
 			return ( newContact != 0L );
 		}
 	}
