@@ -1,9 +1,9 @@
 /*
     msnmessagemanager.cpp - MSN Message Manager
 
-    Copyright (c) 2002 by Olivier Goffart        <ogoffart@tiscalinet.be>
+    Copyright (c) 2002-2004 by Olivier Goffart        <ogoffart@tiscalinet.be>
 
-    Kopete    (c) 2002 by the Kopete developers  <kopete-devel@kde.org>
+    Kopete    (c) 2002-2004 by the Kopete developers  <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -20,6 +20,7 @@
 #include <qlabel.h>
 #include <qimage.h>
 #include <qtooltip.h>
+#include <qfile.h>
 
 #include <kconfig.h>
 #include <kdebug.h>
@@ -28,16 +29,25 @@
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
 #include <ktempfile.h>
+#include <kmainwindow.h>
+#include <ktoolbar.h>
+#include <krun.h>
 
 #include "kopetecontactaction.h"
 #include "kopetecontactlist.h"
 #include "kopetemessagemanagerfactory.h"
+#include "kopeteuiglobal.h"
+#include "kopeteglobal.h"
+#include "kopeteview.h"
 
 #include "msncontact.h"
 #include "msnfiletransfersocket.h"
 #include "msnaccount.h"
 #include "msnswitchboardsocket.h"
 
+#if !defined NDEBUG
+#include "msndebugrawcmddlg.h"
+#endif
 
 MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteContact *user,
 	KopeteContactPtrList others, const char *name )
@@ -47,6 +57,8 @@ MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteCont
 	m_chatService = 0l;
 //	m_msgQueued = 0L;
 
+	setInstance(protocol->instance());
+
 	connect( this, SIGNAL( messageSent( KopeteMessage&,
 		KopeteMessageManager* ) ),
 		this, SLOT( slotMessageSent( KopeteMessage&,
@@ -59,6 +71,10 @@ MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteCont
 	m_actionInvite = new KActionMenu( i18n( "&Invite" ), actionCollection() , "msnInvite" );
 	connect ( m_actionInvite->popupMenu() , SIGNAL( aboutToShow() ) , this , SLOT(slotActionInviteAboutToShow() ) ) ;
 
+	#if !defined NDEBUG
+	new KAction( i18n( "Send Raw C&ommand..." ), 0, this, SLOT( slotDebugRawCommand() ), actionCollection(), "msnDebugRawCommand" ) ;
+	#endif
+
 	MSNContact *c = static_cast<MSNContact*>( others.first() );
 	(new KAction( i18n( "Request Display Picture" ), "image", 0,  this, SLOT( slotRequestPicture() ), actionCollection(), "msnRequestDisplayPicture" ))->setEnabled(!c->object().isEmpty());
 
@@ -66,15 +82,16 @@ MSNMessageManager::MSNMessageManager( KopeteProtocol *protocol, const KopeteCont
 	{
 		connect( c, SIGNAL( displayPictureChanged() ), this, SLOT( slotDisplayPictureChanged() ) );
 		m_image = new QLabel( 0L, "kde toolbar widget" );
-		new KWidgetAction( m_image, i18n( "MSN Display Picture" ), 0, 0, 0, actionCollection(), "msnDisplayPicture" );
-		if ( c->displayPicture() )
+		new KWidgetAction( m_image, i18n( "MSN Display Picture" ), 0, this, SLOT( slotRequestPicture() ), actionCollection(), "msnDisplayPicture" );
+		if(c->displayPicture())
 		{
-			// FIXME: don't duplicate this code with the slotDisplayPictureChanged - Martijn
-			// FIXME: don't hardcode the 22x22 geometry, but adjust to the toolbar's height - Martijn
-			// FIXME: make the tooltip center on the image rather than stay out of the way - Martijn
-			QImage scaledImg = QPixmap( c->displayPicture()->name() ).convertToImage().smoothScale( 22, 22 );
-			m_image->setPixmap( scaledImg );
-			QToolTip::add( m_image, "<qt><img src=\"" + c->displayPicture()->name() + "\"></qt>" );
+			//if the view doesn't exist yet, we will be unable to get the size of the toolbar
+			// so when the view will exist, we will show the displaypicture.
+			//How to know when a our view is created?  We can't.
+			// but chances are the next created view will be for this KMM
+			// And if it is not?  never mind. the icon will just be sized 22x22
+			connect( KopeteMessageManagerFactory::factory() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
+			//it's viewActivated and not viewCreated because the view get his mainwindow only when it is shown.
 		}
 	}
 	else
@@ -106,8 +123,7 @@ void MSNMessageManager::createChat( const QString &handle,
 {
 	if( m_chatService )
 	{
-		kdDebug(14140) << "MSNMessageManager::createChat: "
-			<< "Service already exists, disconnect them." << endl;
+		kdDebug(14140) << k_funcinfo << "Service already exists, disconnect them." << endl;
 		delete m_chatService;
 	}
 
@@ -144,7 +160,7 @@ void MSNMessageManager::slotUserJoined( const QString &handle, const QString &pu
 
 	MSNContact *c = static_cast<MSNContact*>( account()->contacts()[ handle ] );
 
-	if( c->displayName() != publicName)
+	if( c->property( Kopete::Global::Properties::self()->nickName()).value().toString() != publicName)
 		c->rename(publicName);
 
 	addContact(c , IRO); // don't show notificaions when we join wesalef
@@ -153,7 +169,7 @@ void MSNMessageManager::slotUserJoined( const QString &handle, const QString &pu
 
 	KConfig *config = KGlobal::config();
 	config->setGroup( "MSN" );
-	if ( config->readBoolEntry( "AutoDownloadPicture", true ) && !c->object().isEmpty() && !c->displayPicture())
+	if ( members().count()==1 && config->readBoolEntry( "AutoDownloadPicture", true ) && !c->object().isEmpty() && !c->displayPicture())
 		slotRequestPicture();
 }
 
@@ -168,8 +184,8 @@ void MSNMessageManager::slotUserLeft( const QString &handle, const QString& reas
 
 void MSNMessageManager::slotSwitchBoardClosed()
 {
-	kdDebug(14140) << "MSNMessageManager::slotSwitchBoardClosed"  << endl;
-	delete m_chatService; //->deleteLater();
+	//kdDebug(14140) << "MSNMessageManager::slotSwitchBoardClosed"  << endl;
+	m_chatService->deleteLater();
 	m_chatService=0l;
 
 	for ( QMap<unsigned int , KopeteMessage>::iterator it = m_messagesSent.begin(); it!=m_messagesSent.end(); it = m_messagesSent.begin() )
@@ -194,7 +210,12 @@ void MSNMessageManager::slotMessageSent(KopeteMessage &message,KopeteMessageMana
 		if(id == -1)
 		{
 			m_messagesQueue.append(message);
-			kdDebug(14140) << "MSNMessageManager::slotMessageSent: message added to the queue" <<endl;
+			kdDebug(14140) << k_funcinfo << "Message added to the queue" <<endl;
+		}
+		else if( id== -2 ) //the message has not been sent
+		{
+			//FIXME:  tell the what window the message has been processed. but we havent't sent it 
+			messageSucceeded();  //that should stop the blonking icon.
 		}
 		else
 		{
@@ -228,7 +249,7 @@ void MSNMessageManager::slotMessageReceived( KopeteMessage &msg )
 	{
 		KConfig *config = KGlobal::config();
 		config->setGroup( "MSN" );
-		if ( config->readBoolEntry( "SendAwayMessages", true ) &&
+		if ( config->readBoolEntry( "SendAwayMessages", false ) &&
 			( !m_awayMessageTime.isValid() ||
 			m_awayMessageTime.elapsed() > 1000 * config->readNumEntry( "AwayMessagesSeconds", 90 ) )  )
 		{
@@ -255,17 +276,16 @@ void MSNMessageManager::slotActionInviteAboutToShow()
 
 	m_actionInvite->popupMenu()->clear();
 
-	QPtrList<KopeteContact> availableContacts = KopeteContactList::contactList()->onlineContacts( protocol()->pluginId() );
-	QPtrListIterator<KopeteContact> it( availableContacts );
+	
+	QDictIterator<KopeteContact> it( account()->contacts() );
 	for( ; it.current(); ++it )
 	{
-		if( !members().contains( it.current() ) )
+		if( !members().contains( it.current() ) && it.current()->isOnline() && it.current() != user() )
 		{
 			KAction *a=new KopeteContactAction( it.current(), this,
 				SLOT( slotInviteContact( KopeteContact * ) ), m_actionInvite );
 			m_actionInvite->insert( a );
 			m_inviteactions.append( a ) ;
-
 		}
 	}
 	KAction *b=new KAction( i18n ("Other..."), 0, this, SLOT( slotInviteOtherContact() ), m_actionInvite, "actionOther" );
@@ -275,7 +295,7 @@ void MSNMessageManager::slotActionInviteAboutToShow()
 
 void MSNMessageManager::slotCloseSession()
 {
-	kdDebug(14140) << "MSNMessageManager::slotCloseSession: " << m_chatService <<endl;
+	kdDebug(14140) << k_funcinfo  << m_chatService <<endl;
 	if(m_chatService)
 		m_chatService->slotCloseSession();
 }
@@ -300,7 +320,7 @@ void MSNMessageManager::slotInviteOtherContact()
 
 	if( handle.contains('@') != 1 || handle.contains('.') <1)
 	{
-			KMessageBox::queuedMessageBox( 0L, KMessageBox::Sorry,
+			KMessageBox::queuedMessageBox( Kopete::UI::Global::mainWidget(), KMessageBox::Sorry,
 					i18n("<qt>You must enter a valid email address.</qt>"), i18n("MSN Plugin"));
 			return;
 	}
@@ -316,10 +336,10 @@ void MSNMessageManager::sendMessageQueue()
 {
 	if(!m_chatService)
 	{
-		kdDebug(14140) << "MSNMessageManager::sendMessageQueue: service doesn't exist" <<endl;
+		kdDebug(14140) <<k_funcinfo << "Service doesn't exist" <<endl;
 		return;
 	}
-	kdDebug(14140) << "MSNMessageManager::sendMessageQueue: " << m_messagesQueue.count() <<endl;
+//	kdDebug(14140) << "MSNMessageManager::sendMessageQueue: " << m_messagesQueue.count() <<endl;
 	for ( QValueList<KopeteMessage>::iterator it = m_messagesQueue.begin(); it!=m_messagesQueue.end(); it = m_messagesQueue.begin() )
 	{
 		//m_chatService->sendMsg( *it)  ;
@@ -353,6 +373,8 @@ void MSNMessageManager::slotAcknowledgement(unsigned int id, bool ack)
 		QString body = i18n( "The following message has not been sent correctly:\n%1" ).arg( m.plainBody() );
 		KopeteMessage msg = KopeteMessage( m.to().first(), members(), body, KopeteMessage::Internal, KopeteMessage::PlainText );
 		appendMessage( msg );
+		//stop the stupid animation
+		messageSucceeded();  
 	}
 	else
 	{
@@ -406,12 +428,8 @@ void MSNMessageManager::slotInvitation(const QString &handle, const QString &msg
 
 				QString body = i18n(
 					"%1 has sent an unimplemented invitation, the invitation was rejected.\n"
-					"The invitation was: %2" ).
-#if QT_VERSION < 0x030200
-					arg( c->displayName() ).arg( inviteName );
-#else
-					arg( c->displayName(), inviteName );
-#endif
+					"The invitation was: %2" )
+						.arg( c->property( Kopete::Global::Properties::self()->nickName()).value().toString(), inviteName );
 				KopeteMessage tmpMsg = KopeteMessage( c , members() , body , KopeteMessage::Internal, KopeteMessage::PlainText);
 				appendMessage(tmpMsg);
 
@@ -475,14 +493,24 @@ void MSNMessageManager::initInvitation(MSNInvitation* invitation)
 void MSNMessageManager::slotRequestPicture()
 {
 	QPtrList<KopeteContact> mb=members();
-	if(m_chatService)
+	MSNContact *c = static_cast<MSNContact*>( mb.first() );
+	if(!c)
+	 return;
+	
+	if( !c->displayPicture())
 	{
-		MSNContact *c = static_cast<MSNContact*>( mb.first() );
-		if(c && !c->displayPicture() && !c->object().isEmpty() )
-			m_chatService->requestDisplayPicture();
+		if(m_chatService)
+		{
+			if( !c->object().isEmpty() )
+				m_chatService->requestDisplayPicture();
+		}
+		else
+			static_cast<MSNAccount*>( account() )->slotStartChatSession( mb.first()->contactId() );
 	}
 	else
-		static_cast<MSNAccount*>( account() )->slotStartChatSession( mb.first()->contactId() );
+	{ //we already have the picture, just show it.
+		KRun::runURL( KURL::fromPathOrURL( c->displayPicture()->name() ), "image/png" );
+	}
 
 }
 
@@ -491,16 +519,39 @@ void MSNMessageManager::slotDisplayPictureChanged()
 	const MSNContact *c = static_cast<const MSNContact *>( members().getFirst() );
 	if ( c && m_image )
 	{
-		if ( c->displayPicture() )
+		if(c->displayPicture())
 		{
-			// FIXME: don't duplicate this code with the c'tor - Martijn
-			// FIXME: don't hardcode the 22x22 geometry, but adjust to the toolbar's height - Martijn
-			// FIXME: make the tooltip center on the image rather than stay out of the way - Martijn
-			QImage scaledImg = QPixmap( c->displayPicture()->name() ).convertToImage().smoothScale( 22, 22 );
+			int sz=22;
+			// get the size of the toolbar were the aciton is plugged.
+			//  if you know a better way to get the toolbar, let me know
+			KMainWindow *w= view(false) ? dynamic_cast<KMainWindow*>( view(false)->mainWidget()->topLevelWidget() ) : 0L;
+			if(w)
+			{
+				//We connected that in the constructor.  we don't need to keep this slot active.
+				disconnect( KopeteMessageManagerFactory::factory() , SIGNAL(viewActivated(KopeteView* )) , this, SLOT(slotDisplayPictureChanged()) );
+			
+				QPtrListIterator<KToolBar>  it=w->toolBarIterator() ;
+				KAction *imgAction=actionCollection()->action("msnDisplayPicture");
+				if(imgAction)  while(it)
+				{
+					KToolBar *tb=*it;
+					if(imgAction->isPlugged(tb))
+					{
+						sz=tb->iconSize();
+						//ipdate if the size of the toolbar change.
+						disconnect(tb, SIGNAL(modechange()), this, SLOT(slotDisplayPictureChanged()));
+						connect(tb, SIGNAL(modechange()), this, SLOT(slotDisplayPictureChanged()));
+						break;
+					}
+					++it;
+				}
+			}
+			
+			QImage scaledImg = QPixmap( c->displayPicture()->name() ).convertToImage().smoothScale( sz, sz );
 			m_image->setPixmap( scaledImg );
 			QToolTip::add( m_image, "<qt><img src=\"" + c->displayPicture()->name() + "\"></qt>" );
 		}
-		else
+		else 
 		{
 			KConfig *config = KGlobal::config();
 			config->setGroup( "MSN" );
@@ -510,6 +561,23 @@ void MSNMessageManager::slotDisplayPictureChanged()
 	}
 }
 
+void MSNMessageManager::slotDebugRawCommand()
+{
+#if !defined NDEBUG
+	if ( !m_chatService )
+		return;
+
+	MSNDebugRawCmdDlg *dlg = new MSNDebugRawCmdDlg( 0L );
+	int result = dlg->exec();
+	if( result == QDialog::Accepted && m_chatService )
+	{
+		m_chatService->sendCommand( dlg->command(), dlg->params(),
+					dlg->addId(), dlg->msg().replace("\n","\r\n").utf8() );
+	}
+	delete dlg;
+#endif
+}
+
 
 #include "msnmessagemanager.moc"
 
