@@ -2,7 +2,7 @@
  *
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: connect.h,v 1.36 2003/03/07 22:06:51 waba Exp $
+ * $Id: connect.h,v 1.38 2004/04/16 00:56:26 gpuga Exp $
  *
  *            Copyright (C) 1997 Bernd Johannes Wuebben
  *                   wuebben@math.cornell.edu
@@ -30,7 +30,7 @@
 #define _CONNECT_H_
 
 #include <qtimer.h>
-#include <qpushbutton.h>
+#include <kpushbutton.h>
 #include <qlabel.h>
 #include <qevent.h>
 
@@ -54,6 +54,11 @@ public:
   void setMsg(const QString &);
   void pppdDied();
 
+  // these are only necessary to prevent the user from clicking in the cancel 
+  // button during the disconnect delay in KPPPWidget
+  void disableButtons();
+  void enableButtons();
+
 protected:
   void timerEvent(QTimerEvent *);
   void closeEvent( QCloseEvent *e );
@@ -129,7 +134,7 @@ private:
   QTimer *if_timeout_timer;
 
   QLabel *messg;
-  QPushButton *cancel;
+  KPushButton *cancel;
 
   bool firstrunID;
   bool firstrunPW;
